<?php

defined('_JEXEC') OR die('Payzito Joomla Restricted Access');

class PASetupJoomla implements PASetupImplements
{
    static function addScript($dir)
    {
        if(strpos($dir,'.min.js') !== false)
        {
            $file = self::getMediaPath().DS.'scripts'.DS.str_replace('/',DS,substr($dir,0,strpos($dir,'?')));
            if(!file_exists($file))
            {
                $dir = str_replace('.min.js','.js',$dir);
            }
        }

        $url = self::getMediaUri().'scripts/'.$dir;

        $document = JFactory::getDocument();
        $document->addScript($url);
    }

    static function addScriptDeclaration($content)
    {
        $document = JFactory::getDocument();
        $document->addScriptDeclaration($content);
    }

    static function addStyle($dir)
    {
        if(strpos($dir,'.min.css') !== false)
        {
            $file = self::getMediaPath().DS.'styles'.DS.str_replace('/',DS,substr($dir,0,strpos($dir,'?')));
            if(!file_exists($file))
            {
                $dir = str_replace('.min.css','.css',$dir);
            }
        }

        $url = self::getMediaUri().'styles/'.$dir;

        $document = JFactory::getDocument();
        $document->addStyleSheet($url);
    }

    static function getPluginPath($data)
    {
        return self::getBasePath().DS.'plugins'.DS.(isset($data['plg_group']) ? $data['plg_group'] : 'system').DS.$data['plg_name'];
    }

    static function getAdminUri($pathOnly=false)
    {
        return self::getRootUri($pathOnly).'administrator/';
    }

    static function getBasePath()
    {
        return JPATH_SITE;
    }

    static function getRootPath()
    {
        return JPATH_ROOT;
    }

    static function getCorePath()
    {
        return self::getComponentPath().DS.'includes';
    }

    static function getComponentPath()
    {
        return self::getRootPath().DS.'components'.DS.'com_payzito';
    }

    static function getPackagesPath()
    {
        return self::getComponentPath().DS.'packages';
    }

    static function getCoreUri()
    {
        return self::getRootUri().'components/com_payzito/includes/';
    }

    static function getRootUri($pathOnly=false)
    {
        return rtrim(JUri::root($pathOnly),'/').'/';
    }

    static function getBackendUrl($src)
    {
        return 'index.php?option=com_payzito&view='.$src;
    }

    static function setSession($name,$value)
    {
        $session = JFactory::getSession();
        return $session->set($name,$value);
    }

    static function getSession($name)
    {
        $session = JFactory::getSession();
        return $session->get($name);
    }

    static function clearSession($name)
    {
        $session = JFactory::getSession();
        return $session->clear($name);
    }

    static function getSampleAction()
    {
        return [
            'data' => [
                [
                    'PAAction' => 'download',
                    'PAElement' => '{NAME}',
                    'PAFileName' => '{PLUGIN_NAME}.zip',
                ],
                [
                    'PAAction' => 'install',
                    'PAFileName' => '{PLUGIN_NAME}.zip',
                    'PAIsAddon' => 1,
                ]
            ],
            'label' => '{LABEL}',
            'skipIfFail' => true,
        ];
    }

    static function inInstallArea()
    {
        return true;
    }

    static function enableInstallerPlugin()
    {
        $database = JFactory::getDbo();
        $query = "UPDATE `#__extensions` SET `enabled`='1' WHERE `element`='payzito_loader'";
        $database->setQuery($query);
        $database->execute();
    }

    static function getMediaPath()
    {
        return self::getCorePath().DS.'assets';
    }

    static function getMediaUri()
    {
        return self::getCoreUri().'assets/';
    }

    static function moveMediaFiles()
    {
        return false;
    }

    static function moveCoreFiles()
    {
        return false;
    }

    static function movePackageFiles()
    {
        return true;
    }

    static function dbQuery($query)
    {
        $db = JFactory::getDbo();
        $db->setQuery($query);
        $db->execute();
    }

    static function getDomain()
    {
        $uri = JUri::getInstance();
        return $uri->getHost();
    }

    static function tableExist($table)
    {
        $db = JFactory::getDbo();
        $allTables = $db->getTableList();

        $config = JFactory::getConfig();
        $table = str_replace('#__',$config->get('dbprefix'),$table);
        return in_array($table,$allTables) ? true : false;
    }

    static function existAtLeastRow($table,$where='')
    {
        return self::getCountOfTable($table,$where) > 0 ? true : false;
    }

    static function getCountOfTable($table,$where='')
    {
        $db = JFactory::getDbo();
        $db->setQuery("SELECT COUNT(*) AS `count` FROM `".$table."` ".(!empty($where) ? 'WHERE '.$where : ''));
        $db->execute();
        $result = $db->loadAssoc();

        return !empty($result['count']) ? $result['count'] : 0;
    }

    static function isLocalHost()
    {
        $host = self::getDomain();
        return strpos($host,'localhost') !== false || strpos($host,'127.0.0.1') !== false ? true : false;
    }

    static function isValidCmsVersion()
    {
        $status = (bool)version_compare(JVERSION,'3.5.0','>=');
        $msg = !$status ? 'امکان نصب پِی زیتو بر روی نسخه جوملای کمتر از نسخه 3.5.0 وجود ندارد.' : '';

        return [$status,$msg];
    }

    static function getLicenseFilename($type)
    {
        return 'payzito-license-jm'.($type == 'pro' ? 'p' : ($type == 'biz' ? 'b' : 'f')).'.txt';
    }

    static function getUnzipDirectory()
    {
        return null;
    }

    static function getCmsText()
    {
        return 'جوملا';
    }

    static function getCmsSupportedVersions()
    {
        return [3.0,4.0];
    }

    static function getPhpSupportedVersions()
    {
        return [7.4];
    }

    static function isValidPhpVersion()
    {
        $status = version_compare(phpversion(),'7.4.0','>=') && version_compare(phpversion(),'8.2.0','<');
        $msg = !$status ? 'امکان نصب پِی زیتو بر روی این نسخه php وجود ندارد. نسخه php سرور باید حداقل 7.4 و حداکثر 8.1 باشد.' : '';
        return [$status,$msg];
    }

    static function getTempDirectory()
    {
        return self::getPluginPath(['plg_name' => 'payzito_loader']).DS.'payzitotmp';
    }

    static function getOldTempDirectory()
    {
        $config = JFactory::getConfig();
        return $config->get('tmp_path').DS.'payzitotmp';
    }

    static function getSetupSession($key=null)
    {
        $setup = (array)self::getSession('payzito_setup_ready');

        if(!is_null($key))
        {
            return isset($setup[$key]) ? $setup[$key] : false;
        }

        return $setup;
    }

    static function setSetupSession()
    {
        self::setSession('payzito_setup_ready',['start' => 1]);
    }

    static function updateSetupSession($key,$value)
    {
        $data = self::getSetupSession();
        $data[$key] = $value;
        self::setSession('payzito_setup_ready',$data);
    }

    static function clearSetupSession()
    {
        self::clearSession('payzito_setup_ready');
    }

    static function inSetupArea()
    {
        return self::getSetupSession();
    }

    static function checkReadyToUpdate()
    {
        self::dbQuery("UPDATE `#__extensions` SET `enabled` = 1 WHERE `element`='payzito_loader'");
    }
}